@@ -3,10 +3,9 @@ Wrapper methods for easy access to common operations, making them both less
 complex and less verbose for one off or simple use cases.
 
 """
+from rabbitpy import amqp_queue
 from rabbitpy import connection
 from rabbitpy import exchange
-from rabbitpy import exceptions
-from rabbitpy import amqp_queue
 from rabbitpy import message
 
 
@@ -60,33 +59,32 @@ def get(uri=None, queue_name=None):
             return q.get(False)
 
 
-def publish(uri=None, exchange=None, routing_key=None,
+def publish(uri=None, exchange_name=None, routing_key=None,
             body=None, properties=None, confirm=False):
     """Publish a message to RabbitMQ. This should only be used for one-off
     publishing, as you will suffer a performance penalty if you use it
     repeatedly instead creating a connection and channel and publishing on that
 
     :param str uri: AMQP URI to connect to
-    :param str exchange: The exchange to publish to
+    :param str exchange_name: The exchange to publish to
     :param str routing_key: The routing_key to publish with
     :param str or unicode or bytes or dict or list: The message body
     :param dict properties: Dict representation of Basic.Properties
     :param bool confirm: Confirm this delivery with Publisher Confirms
     :rtype: bool or None
-    :raises: :py:class:`ValueError`
 
     """
-    if not exchange:
-        raise ValueError('You must specify an exchange to pubish to')
+    if exchange_name is None:
+        exchange_name = ''
 
     with connection.Connection(uri) as conn:
         with conn.channel() as channel:
             msg = message.Message(channel, body or '', properties or dict())
             if confirm:
                 channel.enable_publisher_confirms()
-                return msg.publish(exchange, routing_key or '')
+                return msg.publish(exchange_name, routing_key or '')
             else:
-                msg.publish(exchange, routing_key or '')
+                msg.publish(exchange_name, routing_key or '')
 
 
 def create_queue(uri=None, queue_name='', durable=True, auto_delete=False,
@@ -137,13 +135,12 @@ def delete_queue(uri=None, queue_name=None):
     :param str uri: AMQP URI to connect to
     :param str queue_name: The queue name to delete
     :rtype: bool
-    :raises: :py:class:`rabbitpy.exceptions.EmptyQueueNameError`
+    :raises: :py:class:`ValueError`
     :raises: :py:class:`rabbitpy.RemoteClosedException`
 
     """
     if not queue_name:
-        raise exceptions.EmptyQueueNameError()
-
+        raise ValueError('You must specify a queue name to consume from')
     with connection.Connection(uri) as conn:
         with conn.channel() as channel:
             queue = amqp_queue.Queue(channel, queue_name)
@@ -157,13 +154,12 @@ def create_direct_exchange(uri=None, exchange_name=None, durable=True):
     :param str uri: AMQP URI to connect to
     :param str exchange_name: The exchange name to create
     :param bool durable: Exchange should survive server restarts
-    :raises: :py:class:`rabbitpy.exceptions.EmptyExchangeNameError`
+    :raises: :py:class:`ValueError`
     :raises: :py:class:`rabbitpy.RemoteClosedException`
 
     """
     if not exchange_name:
-        raise exceptions.EmptyExchangeNameError()
-
+        raise ValueError('You must specify a exchange name to create')
     with connection.Connection(uri) as conn:
         with conn.channel() as channel:
             obj = exchange.DirectExchange(channel, exchange_name,
@@ -178,13 +174,12 @@ def create_fanout_exchange(uri=None, exchange_name=None, durable=True):
     :param str uri: AMQP URI to connect to
     :param str exchange_name: The exchange name to create
     :param bool durable: Exchange should survive server restarts
-    :raises: :py:class:`rabbitpy.exceptions.EmptyExchangeNameError`
+    :raises: :py:class:`ValueError`
     :raises: :py:class:`rabbitpy.RemoteClosedException`
 
     """
     if not exchange_name:
-        raise exceptions.EmptyExchangeNameError()
-
+        raise ValueError('You must specify a exchange name to create')
     with connection.Connection(uri) as conn:
         with conn.channel() as channel:
             obj = exchange.FanoutExchange(channel, exchange_name,
@@ -192,6 +187,26 @@ def create_fanout_exchange(uri=None, exchange_name=None, durable=True):
             obj.declare()
 
 
+def create_headers_exchange(uri=None, exchange_name=None, durable=True):
+    """Create a headers exchange with RabbitMQ. This should only be used for
+    one-off operations.
+
+    :param str uri: AMQP URI to connect to
+    :param str exchange_name: The exchange name to create
+    :param bool durable: Exchange should survive server restarts
+    :raises: :py:class:`ValueError`
+    :raises: :py:class:`rabbitpy.RemoteClosedException`
+
+    """
+    if not exchange_name:
+        raise ValueError('You must specify a exchange name to create')
+    with connection.Connection(uri) as conn:
+        with conn.channel() as channel:
+            obj = exchange.HeadersExchange(channel, exchange_name,
+                                           durable=durable)
+            obj.declare()
+
+
 def create_topic_exchange(uri=None, exchange_name=None, durable=True):
     """Create an exchange from RabbitMQ. This should only be used for one-off
     operations.
@@ -199,13 +214,12 @@ def create_topic_exchange(uri=None, exchange_name=None, durable=True):
     :param str uri: AMQP URI to connect to
     :param str exchange_name: The exchange name to create
     :param bool durable: Exchange should survive server restarts
-    :raises: :py:class:`rabbitpy.exceptions.EmptyExchangeNameError`
+    :raises: :py:class:`ValueError`
     :raises: :py:class:`rabbitpy.RemoteClosedException`
 
     """
     if not exchange_name:
-        raise exceptions.EmptyExchangeNameError()
-
+        raise ValueError('You must specify a exchange name to create')
     with connection.Connection(uri) as conn:
         with conn.channel() as channel:
             obj = exchange.TopicExchange(channel, exchange_name,
@@ -219,13 +233,12 @@ def delete_exchange(uri=None, exchange_name=None):
 
     :param str uri: AMQP URI to connect to
     :param str exchange_name: The exchange name to delete
-    :raises: :py:class:`rabbitpy.exceptions.EmptyExchangeNameError`
+    :raises: :py:class:`ValueError`
     :raises: :py:class:`rabbitpy.RemoteClosedException`
 
     """
     if not exchange_name:
-        raise exceptions.EmptyExchangeNameError()
-
+        raise ValueError('You must specify a exchange name to delete')
     with connection.Connection(uri) as conn:
         with conn.channel() as channel:
             obj = exchange.Exchange(channel, exchange_name)
